@@ -13,6 +13,7 @@
 //!       let entry = binary.entry;
 //!       for ph in binary.program_headers {
 //!         if ph.p_type == goblin::elf::program_header::PT_LOAD {
+//!           // TODO: you should validate p_filesz before allocating.
 //!           let mut _buf = vec![0u8; ph.p_filesz as usize];
 //!           // read responsibly
 //!          }
@@ -37,7 +38,7 @@
 //! `endian_fd` features if you disable `default`.
 
 #[macro_use]
-mod gnu_hash;
+pub(crate) mod gnu_hash;
 
 // These are shareable values for the 32/64 bit implementations.
 //
@@ -48,7 +49,7 @@ pub mod section_header;
 pub mod compression_header;
 #[macro_use]
 pub mod sym;
-pub mod dyn;
+pub mod dynamic;
 #[macro_use]
 pub mod reloc;
 pub mod note;
@@ -61,11 +62,11 @@ macro_rules! if_sylvan {
 }
 
 if_sylvan! {
-    use scroll::{self, ctx, Pread, Endian};
-    use strtab::Strtab;
-    use error;
-    use container::{Container, Ctx};
-    use alloc::vec::Vec;
+    use scroll::{ctx, Pread, Endian};
+    use crate::strtab::Strtab;
+    use crate::error;
+    use crate::container::{Container, Ctx};
+    use crate::alloc::vec::Vec;
     use core::cmp;
 
     pub type Header = header::Header;
@@ -73,8 +74,8 @@ if_sylvan! {
     pub type SectionHeader = section_header::SectionHeader;
     pub type Symtab<'a> = sym::Symtab<'a>;
     pub type Sym = sym::Sym;
-    pub type Dyn = dyn::Dyn;
-    pub type Dynamic = dyn::Dynamic;
+    pub type Dyn = dynamic::Dyn;
+    pub type Dynamic = dynamic::Dynamic;
     pub type Reloc = reloc::Reloc;
     pub type RelocSection<'a> = reloc::RelocSection<'a>;
 
@@ -211,7 +212,7 @@ if_sylvan! {
             if class != header::ELFCLASS64 && class != header::ELFCLASS32 {
                 return Err(error::Error::Malformed(format!("Unknown values in ELF ident header: class: {} endianness: {}",
                                                            class,
-                                                           header.e_ident[header::EI_DATA])).into());
+                                                           header.e_ident[header::EI_DATA])));
             }
             let is_64 = class == header::ELFCLASS64;
             let container = if is_64 { Container::Big } else { Container::Little };
@@ -280,7 +281,7 @@ if_sylvan! {
                 // parse the dynamic relocations
                 dynrelas = RelocSection::parse(bytes, dyn_info.rela, dyn_info.relasz, true, ctx)?;
                 dynrels = RelocSection::parse(bytes, dyn_info.rel, dyn_info.relsz, false, ctx)?;
-                let is_rela = dyn_info.pltrel as u64 == dyn::DT_RELA;
+                let is_rela = dyn_info.pltrel as u64 == dynamic::DT_RELA;
                 pltrelocs = RelocSection::parse(bytes, dyn_info.jmprel, dyn_info.pltrelsz, is_rela, ctx)?;
 
                 let mut num_syms = if let Some(gnu_hash) = dyn_info.gnu_hash {
@@ -300,44 +301,35 @@ if_sylvan! {
                 dynsyms = Symtab::parse(bytes, dyn_info.symtab, num_syms, ctx)?;
             }
 
-            // iterate through shdrs again iff we're an ET_REL
-            let shdr_relocs = {
-                let mut relocs = vec![];
-                if header.e_type == header::ET_REL {
-                    for (idx, section) in section_headers.iter().enumerate() {
-                        if section.sh_type == section_header::SHT_REL {
-                            section.check_size(bytes.len())?;
-                            let sh_relocs = RelocSection::parse(bytes, section.sh_offset as usize, section.sh_size as usize, false, ctx)?;
-                            relocs.push((idx, sh_relocs));
-                        }
-                        if section.sh_type == section_header::SHT_RELA {
-                            section.check_size(bytes.len())?;
-                            let sh_relocs = RelocSection::parse(bytes, section.sh_offset as usize, section.sh_size as usize, true, ctx)?;
-                            relocs.push((idx, sh_relocs));
-                        }
-                    }
+            let mut shdr_relocs = vec![];
+            for (idx, section) in section_headers.iter().enumerate() {
+                let is_rela = section.sh_type == section_header::SHT_RELA;
+                if is_rela || section.sh_type == section_header::SHT_REL {
+                    section.check_size(bytes.len())?;
+                    let sh_relocs = RelocSection::parse(bytes, section.sh_offset as usize, section.sh_size as usize, is_rela, ctx)?;
+                    shdr_relocs.push((idx, sh_relocs));
                 }
-                relocs
-            };
+            }
+
             Ok(Elf {
-                header: header,
-                program_headers: program_headers,
-                section_headers: section_headers,
-                shdr_strtab: shdr_strtab,
-                dynamic: dynamic,
-                dynsyms: dynsyms,
-                dynstrtab: dynstrtab,
-                syms: syms,
-                strtab: strtab,
-                dynrelas: dynrelas,
-                dynrels: dynrels,
-                pltrelocs: pltrelocs,
-                shdr_relocs: shdr_relocs,
-                soname: soname,
-                interpreter: interpreter,
-                libraries: libraries,
-                is_64: is_64,
-                is_lib: is_lib,
+                header,
+                program_headers,
+                section_headers,
+                shdr_strtab,
+                dynamic,
+                dynsyms,
+                dynstrtab,
+                syms,
+                strtab,
+                dynrelas,
+                dynrels,
+                pltrelocs,
+                shdr_relocs,
+                soname,
+                interpreter,
+                libraries,
+                is_64,
+                is_lib,
                 entry: entry as u64,
                 little_endian: is_lsb,
                 ctx,
@@ -346,9 +338,8 @@ if_sylvan! {
     }
 
     impl<'a> ctx::TryFromCtx<'a, (usize, Endian)> for Elf<'a> {
-        type Error = ::error::Error;
-        type Size = usize;
-        fn try_from_ctx(src: &'a [u8], (_, _): (usize, Endian)) -> Result<(Elf<'a>, Self::Size), Self::Error> {
+        type Error = crate::error::Error;
+        fn try_from_ctx(src: &'a [u8], (_, _): (usize, Endian)) -> Result<(Elf<'a>, usize), Self::Error> {
             let elf = Elf::parse(src)?;
             Ok((elf, src.len()))
         }
@@ -413,22 +404,19 @@ mod tests {
                 assert!(binary.syms.get(1000).is_none());
                 assert!(binary.syms.get(5).is_some());
                 let syms = binary.syms.to_vec();
-                let mut i = 0;
-                assert!(binary.section_headers.len() != 0);
-                for sym in &syms {
+                assert!(!binary.section_headers.is_empty());
+                for (i, sym) in syms.iter().enumerate() {
                     if i == 11 {
                         let symtab = binary.strtab;
                         println!("sym: {:?}", &sym);
                         assert_eq!(&symtab[sym.st_name], "_start");
                         break;
                     }
-                    i += 1;
                 }
-                assert!(syms.len() != 0);
+                assert!(!syms.is_empty());
              },
             Err (err) => {
-                println!("failed: {}", err);
-                assert!(false)
+                panic!("failed: {}", err);
             }
         }
     }
@@ -444,22 +432,19 @@ mod tests {
                 assert!(binary.syms.get(1000).is_none());
                 assert!(binary.syms.get(5).is_some());
                 let syms = binary.syms.to_vec();
-                let mut i = 0;
-                assert!(binary.section_headers.len() != 0);
-                for sym in &syms {
+                assert!(!binary.section_headers.is_empty());
+                for (i, sym) in syms.iter().enumerate() {
                     if i == 11 {
                         let symtab = binary.strtab;
                         println!("sym: {:?}", &sym);
                         assert_eq!(&symtab[sym.st_name], "__libc_csu_fini");
                         break;
                     }
-                    i += 1;
                 }
-                assert!(syms.len() != 0);
+                assert!(!syms.is_empty());
              },
             Err (err) => {
-                println!("failed: {}", err);
-                assert!(false)
+                panic!("failed: {}", err);
             }
         }
     }
