@@ -1,11 +1,13 @@
-use scroll::{self, Pread};
-use alloc::vec::Vec;
+use scroll::{Pread, Pwrite};
+use crate::alloc::vec::Vec;
 
-use error;
+use log::debug;
 
-use pe::utils;
-use pe::section_table;
-use pe::data_directories;
+use crate::error;
+
+use crate::pe::utils;
+use crate::pe::section_table;
+use crate::pe::data_directories;
 
 #[repr(C)]
 #[derive(Debug, PartialEq, Copy, Clone, Default)]
@@ -64,7 +66,7 @@ pub struct ExportData<'a> {
 }
 
 impl<'a> ExportData<'a> {
-    pub fn parse(bytes: &'a [u8], dd: &data_directories::DataDirectory, sections: &[section_table::SectionTable], file_alignment: u32) -> error::Result<ExportData<'a>> {
+    pub fn parse(bytes: &'a [u8], dd: data_directories::DataDirectory, sections: &[section_table::SectionTable], file_alignment: u32) -> error::Result<ExportData<'a>> {
         let export_rva = dd.virtual_address as usize;
         let size = dd.size as usize;
         debug!("export_rva {:#x} size {:#}", export_rva, size);
@@ -127,11 +129,11 @@ impl<'a> ExportData<'a> {
         let name = utils::find_offset(export_directory_table.name_rva as usize, sections, file_alignment).and_then(|offset| bytes.pread(offset).ok());
 
         Ok(ExportData {
-            name: name,
-            export_directory_table: export_directory_table,
-            export_name_pointer_table: export_name_pointer_table,
-            export_ordinal_table: export_ordinal_table,
-            export_address_table: export_address_table,
+            name,
+            export_directory_table,
+            export_name_pointer_table,
+            export_ordinal_table,
+            export_address_table,
         })
     }
 }
@@ -140,48 +142,36 @@ impl<'a> ExportData<'a> {
 /// PE binaries have two kinds of reexports, either specifying the dll's name, or the ordinal value of the dll
 pub enum Reexport<'a> {
   DLLName { export: &'a str, lib: &'a str },
-  DLLOrdinal { export: &'a str, ordinal: usize }
+  DLLOrdinal { ordinal: usize, lib: &'a str }
 }
 
 impl<'a> scroll::ctx::TryFromCtx<'a, scroll::Endian> for Reexport<'a> {
-    type Error = ::error::Error;
-    type Size = usize;
+    type Error = crate::error::Error;
     #[inline]
-    fn try_from_ctx(bytes: &'a [u8], _ctx: scroll::Endian) -> Result<(Self, Self::Size), Self::Error> {
-        use scroll::{Pread};
+    fn try_from_ctx(bytes: &'a [u8], _ctx: scroll::Endian) -> Result<(Self, usize), Self::Error> {
         let reexport = bytes.pread::<&str>(0)?;
         let reexport_len = reexport.len();
         debug!("reexport: {}", &reexport);
         for o in 0..reexport_len {
             let c: u8 = bytes.pread(o)?;
             debug!("reexport offset: {:#x} char: {:#x}", o, c);
-            match c {
-                // '.'
-                0x2e => {
-                    let i = o - 1;
-                    let dll: &'a str = bytes.pread_with(0, ::scroll::ctx::StrCtx::Length(i))?;
-                    debug!("dll: {:?}", &dll);
-                    let len = reexport_len - i - 1;
-                    let rest: &'a [u8] = bytes.pread_with(o, len)?;
-                    debug!("rest: {:?}", &rest);
-                    let len = rest.len() - 1;
-                    match rest[0] {
-                        // '#'
-                        0x23 => {
-                            // UNTESTED
-                            let ordinal = rest.pread_with::<&str>(1, ::scroll::ctx::StrCtx::Length(len))?;
-                            let ordinal = ordinal.parse::<u32>().map_err(|_e| error::Error::Malformed(format!("Cannot parse reexport ordinal from {} bytes", bytes.len())))?;
-                            // FIXME: return size
-                            return Ok((Reexport::DLLOrdinal { export: dll, ordinal: ordinal as usize }, 0))
-                        },
-                        _ => {
-                            let export = rest.pread_with::<&str>(1, ::scroll::ctx::StrCtx::Length(len))?;
-                            // FIXME: return size
-                            return Ok((Reexport::DLLName { export: export, lib: dll }, 0))
-                        }
-                    }
-                },
-                _ => {}
+            if c == b'.' {
+                let dll: &'a str = bytes.pread_with(0, scroll::ctx::StrCtx::Length(o))?;
+                debug!("dll: {:?}", &dll);
+                if o + 1 == reexport_len {
+                    break;
+                }
+                let len = reexport_len - o - 1;
+                let rest: &'a [u8] = bytes.pread_with(o + 1, len)?;
+                debug!("rest: {:?}", &rest);
+                if rest[0] == b'#' {
+                    let ordinal = rest.pread_with::<&str>(1, scroll::ctx::StrCtx::Length(len - 1))?;
+                    let ordinal = ordinal.parse::<u32>().map_err(|_e| error::Error::Malformed(format!("Cannot parse reexport ordinal from {} bytes", bytes.len())))?;
+                    return Ok((Reexport::DLLOrdinal { ordinal: ordinal as usize, lib: dll }, reexport_len + 1))
+                } else {
+                    let export = rest.pread_with::<&str>(0, scroll::ctx::StrCtx::Length(len))?;
+                    return Ok((Reexport::DLLName { export, lib: dll }, reexport_len + 1))
+                }
             }
         }
         Err(error::Error::Malformed(format!("Reexport {:#} is malformed", reexport)))
@@ -189,7 +179,7 @@ impl<'a> scroll::ctx::TryFromCtx<'a, scroll::Endian> for Reexport<'a> {
 }
 
 impl<'a> Reexport<'a> {
-    pub fn parse(bytes: &'a [u8], offset: usize) -> ::error::Result<Reexport<'a>> {
+    pub fn parse(bytes: &'a [u8], offset: usize) -> crate::error::Result<Reexport<'a>> {
         bytes.pread(offset)
     }
 }
@@ -216,12 +206,11 @@ struct ExportCtx<'a> {
 
 impl<'a, 'b> scroll::ctx::TryFromCtx<'a, ExportCtx<'b>> for Export<'a> {
     type Error = error::Error;
-    type Size = usize;
     #[inline]
-    fn try_from_ctx(bytes: &'a [u8], ExportCtx { ptr, idx, sections, file_alignment, addresses, ordinals }: ExportCtx<'b>) -> Result<(Self, Self::Size), Self::Error> {
+    fn try_from_ctx(bytes: &'a [u8], ExportCtx { ptr, idx, sections, file_alignment, addresses, ordinals }: ExportCtx<'b>) -> Result<(Self, usize), Self::Error> {
         use self::ExportAddressTableEntry::*;
 
-        let name = utils::find_offset(ptr as usize, sections, file_alignment).map_or(None, |offset| bytes.pread::<&str>(offset).ok());
+        let name = utils::find_offset(ptr as usize, sections, file_alignment).and_then(|offset| bytes.pread::<&str>(offset).ok());
 
         if let Some(ordinal) = ordinals.get(idx) {
             if let Some(rva) = addresses.get(*ordinal as usize) {
@@ -258,7 +247,7 @@ impl<'a> Export<'a> {
         for (idx, &ptr) in pointers.iter().enumerate() {
             if let Ok(export) = bytes.pread_with(0, ExportCtx { ptr, idx, sections, file_alignment, addresses, ordinals }) {
                 exports.push(export);
-            }    
+            }
         }
 
         // TODO: sort + compute size
